@@ -139,6 +139,7 @@ static void gconf_client_queue_notify       (GConfClient *client,
                                              const char  *key);
 static void gconf_client_flush_notifies     (GConfClient *client);
 static void gconf_client_unqueue_notifies   (GConfClient *client);
+static void notify_one_entry (GConfClient *client, GConfEntry  *entry);
 
 static GConfEntry* get (GConfClient  *client,
                         const gchar  *key,
@@ -339,8 +340,8 @@ gconf_client_real_unreturned_error (GConfClient* client, GError* error)
               error->code == GCONF_ERROR_NO_WRITABLE_DATABASE)
             return;
           
-          fprintf (stderr, _("GConf Error: %s\n"),
-                   error->message);
+          g_printerr (_("GConf Error: %s\n"),
+                      error->message);
         }
     }
 }
@@ -357,8 +358,8 @@ gconf_client_real_error            (GConfClient* client, GError* error)
         }
       else
         {
-          fprintf (stderr, _("GConf Error: %s\n"),
-                   error->message);
+          g_printerr (_("GConf Error: %s\n"),
+                      error->message);
         }
     }
 }
@@ -745,6 +746,22 @@ gconf_client_notify_remove  (GConfClient* client,
     }
 }
 
+void
+gconf_client_notify (GConfClient* client, const char* key)
+{
+  GConfEntry *entry;
+
+  g_return_if_fail (client != NULL);
+  g_return_if_fail (GCONF_IS_CLIENT(client));
+  g_return_if_fail (key != NULL);
+
+  entry = gconf_client_get_entry (client, key, NULL, TRUE, NULL);
+  if (entry != NULL)
+    {
+      notify_one_entry (client, entry);
+    }
+}
+
 void
 gconf_client_set_error_handling(GConfClient* client,
                                 GConfClientErrorHandlingMode mode)
@@ -782,7 +799,7 @@ static void
 cache_pairs_in_dir(GConfClient* client, const gchar* path);
 
 static void 
-recurse_subdir_list(GConfClient* client, GSList* subdirs, const gchar* parent)
+recurse_subdir_list(GConfClient* client, GSList* subdirs)
 {
   GSList* tmp;
 
@@ -791,18 +808,15 @@ recurse_subdir_list(GConfClient* client, GSList* subdirs, const gchar* parent)
   while (tmp != NULL)
     {
       gchar* s = tmp->data;
-      gchar* full = gconf_concat_dir_and_key(parent, s);
       
-      cache_pairs_in_dir(client, full);
+      cache_pairs_in_dir(client, s);
 
       PUSH_USE_ENGINE (client);
       recurse_subdir_list(client,
-                          gconf_engine_all_dirs (client->engine, full, NULL),
-                          full);
+                          gconf_engine_all_dirs (client->engine, s, NULL));
       POP_USE_ENGINE (client);
 
       g_free(s);
-      g_free(full);
       
       tmp = g_slist_next(tmp);
     }
@@ -810,22 +824,6 @@ recurse_subdir_list(GConfClient* client, GSList* subdirs, const gchar* parent)
   g_slist_free(subdirs);
 }
 
-
-typedef struct
-{
-  gboolean found_parent;
-  const char *key;
-} CheckUnderneathData;
-
-static void
-foreach_check_underneath (gpointer key, gpointer value, gpointer user_data)
-{
-  CheckUnderneathData *cud = user_data;
-  
-  if (gconf_key_is_below (key, cud->key))
-    cud->found_parent = TRUE;
-}
-
 static gboolean
 key_being_monitored (GConfClient *client,
                      const char  *key)
@@ -894,8 +892,8 @@ cache_pairs_in_dir(GConfClient* client, const gchar* dir)
   
   if (error != NULL)
     {
-      fprintf(stderr, _("GConf warning: failure listing pairs in `%s': %s"),
-              dir, error->message);
+      g_printerr (_("GConf warning: failure listing pairs in `%s': %s"),
+                  dir, error->message);
       g_error_free(error);
       error = NULL;
     }
@@ -931,14 +929,8 @@ gconf_client_preload    (GConfClient* client,
 
     case GCONF_CLIENT_PRELOAD_ONELEVEL:
       {
-        GSList* subdirs;
-
         trace ("Onelevel preload of '%s'\n", dirname);
         
-        PUSH_USE_ENGINE (client);
-        subdirs = gconf_engine_all_dirs (client->engine, dirname, NULL);
-        POP_USE_ENGINE (client);
-        
         cache_pairs_in_dir (client, dirname);
       }
       break;
@@ -955,7 +947,7 @@ gconf_client_preload    (GConfClient* client,
         
         cache_pairs_in_dir(client, dirname);
           
-        recurse_subdir_list(client, subdirs, dirname);
+        recurse_subdir_list(client, subdirs);
       }
       break;
 
@@ -1005,6 +997,28 @@ gconf_client_unset          (GConfClient* client,
     return TRUE;
 }
 
+gboolean
+gconf_client_recursive_unset (GConfClient *client,
+                              const char     *key,
+                              GConfUnsetFlags flags,
+                              GError        **err)
+{
+  GError* error = NULL;
+
+  trace ("Unsetting '%s'\n", key);
+  
+  PUSH_USE_ENGINE (client);
+  gconf_engine_recursive_unset(client->engine, key, flags, &error);
+  POP_USE_ENGINE (client);
+  
+  handle_error(client, error, err);
+
+  if (error != NULL)
+    return FALSE;
+  else
+    return TRUE;
+}
+
 static GSList*
 copy_entry_list (GSList *list)
 {
@@ -1121,7 +1135,8 @@ gconf_client_key_is_writable (GConfClient* client,
   GError* error = NULL;
   GConfEntry *entry = NULL;
   gboolean is_writable;
-  
+
+  g_return_val_if_fail (key != NULL, FALSE);
   g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
 
   trace ("Checking whether key '%s' is writable... \n", key);
@@ -1140,7 +1155,10 @@ gconf_client_key_is_writable (GConfClient* client,
   else
     g_assert (error == NULL);
 
-  is_writable = gconf_entry_get_is_writable (entry);  
+  if (entry == NULL)
+    is_writable = FALSE;
+  else
+    is_writable = gconf_entry_get_is_writable (entry);  
 
   if (entry)
     gconf_entry_free (entry);
@@ -1297,7 +1315,9 @@ GConfValue*
 gconf_client_get             (GConfClient* client,
                               const gchar* key,
                               GError** err)
-{  
+{
+  g_return_val_if_fail (GCONF_IS_CLIENT (client), NULL);
+  g_return_val_if_fail (key != NULL, NULL);
   return gconf_client_get_full (client, key, NULL, TRUE, err);
 }
 
@@ -1306,6 +1326,8 @@ gconf_client_get_without_default  (GConfClient* client,
                                    const gchar* key,
                                    GError** err)
 {
+  g_return_val_if_fail (GCONF_IS_CLIENT (client), NULL);
+  g_return_val_if_fail (key != NULL, NULL);
   return gconf_client_get_full (client, key, NULL, FALSE, err);
 }
 
@@ -1321,7 +1343,8 @@ gconf_client_get_default_from_schema (GConfClient* client,
   g_return_val_if_fail (err == NULL || *err == NULL, NULL);  
   g_return_val_if_fail (client != NULL, NULL);
   g_return_val_if_fail (GCONF_IS_CLIENT(client), NULL);
-
+  g_return_val_if_fail (key != NULL, NULL);
+  
   trace ("Getting default for %s from schema\n", key);
   
   /* Check our client-side cache to see if the default is the same as
@@ -1437,7 +1460,6 @@ gchar*
 gconf_client_get_string(GConfClient* client, const gchar* key,
                         GError** err)
 {
-  static const gchar* def = NULL;
   GError* error = NULL;
   GConfValue* val;
 
@@ -1452,16 +1474,9 @@ gconf_client_get_string(GConfClient* client, const gchar* key,
       g_assert(error == NULL);
       
       if (check_type (key, val, GCONF_VALUE_STRING, &error))
-        /* we cheat here (look below) so we have to cast this */
-        retval = (gchar *)gconf_value_get_string (val);
+        retval = gconf_value_steal_string (val);
       else
         handle_error (client, error, err);
-
-      /* This is a cheat; don't copy */
-      if (retval != NULL)
-        val->d.string_data = NULL; /* don't delete the string we are returning */
-      else
-        retval = def ? g_strdup (def) : NULL;
       
       gconf_value_free (val);
 
@@ -1471,7 +1486,7 @@ gconf_client_get_string(GConfClient* client, const gchar* key,
     {
       if (error != NULL)
         handle_error (client, error, err);
-      return def ? g_strdup (def) : NULL;
+      return NULL;
     }
 }
 
@@ -1529,13 +1544,9 @@ gconf_client_get_schema  (GConfClient* client,
       g_assert(error == NULL);
       
       if (check_type (key, val, GCONF_VALUE_SCHEMA, &error))
-        retval = gconf_value_get_schema (val);
+        retval = gconf_value_steal_schema (val);
       else
         handle_error (client, error, err);
-
-      /* This is a cheat; don't copy */
-      if (retval != NULL)
-        val->d.schema_data = NULL; /* don't delete the schema */
       
       gconf_value_free (val);
 
@@ -2339,8 +2350,8 @@ notify_listeners_callback(GConfListeners* listeners,
                           gpointer listener_data,
                           gpointer user_data)
 {
-  struct ClientAndEntry* cae = user_data;
   Listener* l = listener_data;
+  struct ClientAndEntry* cae = user_data;
   
   g_return_if_fail (cae != NULL);
   g_return_if_fail (cae->client != NULL);
@@ -2355,12 +2366,15 @@ static void
 notify_one_entry (GConfClient *client,
                   GConfEntry  *entry)
 {
+  g_object_ref (G_OBJECT (client));
+  gconf_entry_ref (entry);
+  
   /* Emit the value_changed signal before notifying specific listeners;
    * I'm not sure there's a reason this matters though
    */
   gconf_client_value_changed (client,
                               entry->key,
-                              entry->value);
+                              gconf_entry_get_value (entry));
 
   /* Now notify our listeners, if any */
   if (client->listeners != NULL)
@@ -2375,6 +2389,9 @@ notify_one_entry (GConfClient *client,
                               notify_listeners_callback,
                               &cae);
     }
+
+  gconf_entry_unref (entry);
+  g_object_unref (G_OBJECT (client));
 }
 
 static void
